package fluent_test

import (
	"testing"

	. "github.com/warpfork/go-wish"

	ld "gitlab.dms3.io/ld/go-ld-prime"
	"gitlab.dms3.io/ld/go-ld-prime/fluent"
	basicnode "gitlab.dms3.io/ld/go-ld-prime/node/basic"
)

func TestRecover(t *testing.T) {
	t.Run("simple build error should capture", func(t *testing.T) {
		Wish(t,
			fluent.Recover(func() {
				fluent.MustBuild(basicnode.Prototype__String{}, func(fna fluent.NodeAssembler) {
					fna.AssignInt(9)
				})
				t.Fatal("should not be reached")
			}),
			ShouldEqual,
			fluent.Error{ld.ErrWrongKind{TypeName: "string", MethodName: "AssignInt", AppropriateKind: ld.KindSet_JustInt, ActualKind: ld.Kind_String}},
		)
	})
	t.Run("correct build should return nil", func(t *testing.T) {
		Wish(t,
			fluent.Recover(func() {
				fluent.MustBuild(basicnode.Prototype__String{}, func(fna fluent.NodeAssembler) {
					fna.AssignString("fine")
				})
			}),
			ShouldEqual,
			nil,
		)
	})
	t.Run("other panics should continue to rise", func(t *testing.T) {
		Wish(t,
			func() (r interface{}) {
				defer func() { r = recover() }()
				fluent.Recover(func() {
					panic("fuqawds")
				})
				return
			}(),
			ShouldEqual,
			"fuqawds",
		)
	})
}
