package basicnode

import (
	ld "gitlab.dms3.io/ld/go-ld-prime"
	"gitlab.dms3.io/ld/go-ld-prime/node/mixins"
)

var (
	_ ld.Node          = plainBytes(nil)
	_ ld.NodePrototype = Prototype__Bytes{}
	_ ld.NodeBuilder   = &plainBytes__Builder{}
	_ ld.NodeAssembler = &plainBytes__Assembler{}
)

func NewBytes(value []byte) ld.Node {
	v := plainBytes(value)
	return &v
}

// plainBytes is a simple boxed byte slice that complies with ld.Node.
type plainBytes []byte

// -- Node interface methods -->

func (plainBytes) Kind() ld.Kind {
	return ld.Kind_Bytes
}
func (plainBytes) LookupByString(string) (ld.Node, error) {
	return mixins.Bytes{TypeName: "bytes"}.LookupByString("")
}
func (plainBytes) LookupByNode(key ld.Node) (ld.Node, error) {
	return mixins.Bytes{TypeName: "bytes"}.LookupByNode(nil)
}
func (plainBytes) LookupByIndex(idx int64) (ld.Node, error) {
	return mixins.Bytes{TypeName: "bytes"}.LookupByIndex(0)
}
func (plainBytes) LookupBySegment(seg ld.PathSegment) (ld.Node, error) {
	return mixins.Bytes{TypeName: "bytes"}.LookupBySegment(seg)
}
func (plainBytes) MapIterator() ld.MapIterator {
	return nil
}
func (plainBytes) ListIterator() ld.ListIterator {
	return nil
}
func (plainBytes) Length() int64 {
	return -1
}
func (plainBytes) IsAbsent() bool {
	return false
}
func (plainBytes) IsNull() bool {
	return false
}
func (plainBytes) AsBool() (bool, error) {
	return mixins.Bytes{TypeName: "bytes"}.AsBool()
}
func (plainBytes) AsInt() (int64, error) {
	return mixins.Bytes{TypeName: "bytes"}.AsInt()
}
func (plainBytes) AsFloat() (float64, error) {
	return mixins.Bytes{TypeName: "bytes"}.AsFloat()
}
func (plainBytes) AsString() (string, error) {
	return mixins.Bytes{TypeName: "bytes"}.AsString()
}
func (n plainBytes) AsBytes() ([]byte, error) {
	return []byte(n), nil
}
func (plainBytes) AsLink() (ld.Link, error) {
	return mixins.Bytes{TypeName: "bytes"}.AsLink()
}
func (plainBytes) Prototype() ld.NodePrototype {
	return Prototype__Bytes{}
}

// -- NodePrototype -->

type Prototype__Bytes struct{}

func (Prototype__Bytes) NewBuilder() ld.NodeBuilder {
	var w plainBytes
	return &plainBytes__Builder{plainBytes__Assembler{w: &w}}
}

// -- NodeBuilder -->

type plainBytes__Builder struct {
	plainBytes__Assembler
}

func (nb *plainBytes__Builder) Build() ld.Node {
	return nb.w
}
func (nb *plainBytes__Builder) Reset() {
	var w plainBytes
	*nb = plainBytes__Builder{plainBytes__Assembler{w: &w}}
}

// -- NodeAssembler -->

type plainBytes__Assembler struct {
	w *plainBytes
}

func (plainBytes__Assembler) BeginMap(sizeHint int64) (ld.MapAssembler, error) {
	return mixins.BytesAssembler{TypeName: "bytes"}.BeginMap(0)
}
func (plainBytes__Assembler) BeginList(sizeHint int64) (ld.ListAssembler, error) {
	return mixins.BytesAssembler{TypeName: "bytes"}.BeginList(0)
}
func (plainBytes__Assembler) AssignNull() error {
	return mixins.BytesAssembler{TypeName: "bytes"}.AssignNull()
}
func (plainBytes__Assembler) AssignBool(bool) error {
	return mixins.BytesAssembler{TypeName: "bytes"}.AssignBool(false)
}
func (plainBytes__Assembler) AssignInt(int64) error {
	return mixins.BytesAssembler{TypeName: "bytes"}.AssignInt(0)
}
func (plainBytes__Assembler) AssignFloat(float64) error {
	return mixins.BytesAssembler{TypeName: "bytes"}.AssignFloat(0)
}
func (plainBytes__Assembler) AssignString(string) error {
	return mixins.BytesAssembler{TypeName: "bytes"}.AssignString("")
}
func (na *plainBytes__Assembler) AssignBytes(v []byte) error {
	*na.w = plainBytes(v)
	return nil
}
func (plainBytes__Assembler) AssignLink(ld.Link) error {
	return mixins.BytesAssembler{TypeName: "bytes"}.AssignLink(nil)
}
func (na *plainBytes__Assembler) AssignNode(v ld.Node) error {
	if v2, err := v.AsBytes(); err != nil {
		return err
	} else {
		*na.w = plainBytes(v2)
		return nil
	}
}
func (plainBytes__Assembler) Prototype() ld.NodePrototype {
	return Prototype__Bytes{}
}
